
/*
 *     Zadatak #13
 *  Slijepcevic Dusan
 *  dusans@drenik.net
 *      367/2000
 */

#include <stdio.h>
#include <conio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "atype.h"
#include "options.h"
#include "misc.h"

const byte logo[80] =
/* 01234567890123456789012345678901234567890123456789012345678901234567890123456789 */
  "atype v1.17b   Copyright (c) 2001. by -=KKF=-    mode: t b h  page: 00001/00001 ";

/*  zbog Watcom kompajlera ovo su konstante (koriste se u `#pragma aux' asm blokovima,
    pod dvostrukim navodnicima, pa preprocesor ne moze da zameni vrednosti) */
const byte TITLE_CLR = BACK_RED | FORE_YELLOW;
const byte TITLE_LEN = 80;

/*----------------------------------------------------------.
|  malloc() wrapper.                                        |
|                                                           |
|  Napomena:                                                |
|   Watcom kompajler vraca NULL ako je n==0, dok vecina     |
|   kompajlera vraca pointer na blok nulte velicine, s tim  |
|   sto je header bloka, naravno, ispravno podesen, pa ima  |
|   smisla osloboditi tu memoriju.                          |
`----------------------------------------------------------*/

void *xmalloc(size_t n)
{
    void *p;
    if ((p = malloc(n
#ifdef __WATCOMC__
        ? n : 1
#endif
        )) == NULL)
        error_me("Memory exhausted!");
    return p;
}

/*-----------------.
|  Error handler.  |
`-----------------*/

void error_me(const char *fmt, ...)
{
    va_list args;

    atype_settextpos(0, 0);
    atype_showcursor(CUR_ON);
    atype_clearscreen();

    va_start(args, fmt);
    vfprintf(stderr, fmt, args);
    fputc('\n',stderr);
    va_end(args);

    exit(EXIT_FAILURE);
}

/*------------------------------.
|  Dodavanje stranice u listu.  |
`------------------------------*/

page *add_page(page *last, long offset, unsigned cnt)
{
    page *newpage = (page *) xmalloc(sizeof(page));

    newpage->offset = offset;  /* file offset pocetka stranice */
    newpage->cnt = cnt;        /* broj `sirovih' (raw) bajtova */
    newpage->prev = last;      /* pointer na poslednju stranicu (ili NULL) */
    newpage->next = NULL;

    if (last) last->next = newpage;
    return newpage;
}

/*------------------------------.
|  Brisanje liste iz memorije.  |
`------------------------------*/

void delete_pagelist(page *first)
{
    page *pg;
    for (; first; first = pg) {
        pg = first->next;
        free(first);
    }
}

/*----------------------------------------------------------------------.
|  Ucitavanje i prikazivanje (`crtanje') stranice na virtuelnom ekranu. |
|                                                                       |
|  Parametri:                                                           |
|    - *file:  fajl koji se cita                                        |
|    - *pg:    ptr na strukturu sa podacima o strani koja se prikazuje  |
|    - *vscr:  virtuelni ekran velicine MAX_CHARS po kom se `crta'      |
|    - mode:   rezim `crtanja': MODE_TEXT ili MODE_BIN                  |
|                                                                       |
|  Napomena:                                                            |
|    Da bi ova f-ja radila kako treba, neophodno je da broj bajtova     |
|    potrebnih za stranicu (pg->cnt) bude ispravno podesen! U suprotnom |
|    moze doci do brisanja (prepisivanja) delova memorije! (petlje ne   |
|    testiraju *vscr ptr!)                                              |
`----------------------------------------------------------------------*/

void load_page(FILE *file, const page *pg, byte *vscr, int mode)
{
    unsigned cnt = pg->cnt;
    byte *buf;    /* buffer velicine pg->cnt bajtova */
    unsigned i=0; /* indeksiranje je prirodnije jer se sam index dosta testira */

    memset(vscr, ' ', MAX_CHARS); /* prvo obrisi ceo virtuelni ekran */
    if (cnt == 0)
        return;   /* stranica je prazna */

    buf = (byte *) xmalloc(cnt * sizeof(byte));

    /* fseek vraca 0 ako je sve proslo OK ! */
    if (fseek(file, pg->offset, SEEK_SET))
        error_me("Error seeking in file!");

    if (fread(buf, sizeof(byte), cnt, file) != cnt)
        error_me("Error reading file!");

    while (cnt--) {
        switch (*buf) {

          case '\r':
              if (cnt && buf[1] == '\n') {
                  i += SCR_COLS - i % SCR_COLS; /* predji u prvu kolonu sledeceg reda */
                  buf += 2;     /* povecaj, procitali smo i <LF> */
                  --cnt;        /* smanji broj preostalih bajtova (testiranje je na vrhu) */
                  continue;     /* i odmah nastavi petlju (preskace se i ++buf) */
              } else {
                  if (mode == MODE_TEXT)
                      i -= i % SCR_COLS; /* vrati se na pocetak reda */
                  else
                      vscr[i++] = *buf; /* u BIN modu ispisi jedan karakter */
              }
              break;

          case '\n':
    #if LF_IS_NEWLINE
              i += SCR_COLS - i % SCR_COLS; /* predji u prvu kolonu sledeceg reda */
    #else
              if (mode == MODE_TEXT)
                  i += SCR_COLS;  /* predji u sledeci red (broj kolone se ne menja!) */
              else
                  vscr[i++] = *buf;  /* u BIN modu ispisi jedan karakter */
    #endif
              break;

          case '\t':
          {
              int indent = TABSIZE - i % TABSIZE;
    #if TABSIZE != 8
              if ((i % SCR_COLS + indent) > SCR_COLS)
                  i += SCR_COLS - i % SCR_COLS; /* predji u sledeci red (prva kolona) */
              else
    #endif
                  i += indent;

              /* ako su vrednosti u gornjem izrazu jednake, onda ce
                 vscr[i+=indent] biti prva celija u sledecem redu */
          }
          break;

    #if RING_A_BELL
          case '\a':
              if (mode == MODE_TEXT)
                  putch('\a');  /* zazvoni! (bez prikazivanja karaktera) */
              else
                  vscr[i++] = *buf; /* u BIN modu ispisi kao karakter */
              break;
    #endif

          case '\b':
              if (mode == MODE_TEXT) {
                  if (i % SCR_COLS)  /* ako nismo na pocetku reda */
                      --i;           /* vrati kursor za jedno mesto ulevo */
                  break;             /* i preskoci ispisivanje */
              }
              /* u BIN modu samo ispisi karakter (ovde nema break-a) */

          default:
              vscr[i++] = *buf;
              break;
        }                       /* end switch */

        ++buf;                  /* citaj `cnt' bajtova iz buffer-a */
    }                           /* end while */

    free(buf - pg->cnt);
}

/*-----------------------------------------------------------------.
|  Ucitavanje i prikazivanje hex dump-a na virtuelnom ekranu.      |
|                                                                  |
|  Parametri:                                                      |
|    - *file:  fajl koji se cita                                   |
|    - offset: file offset stranice                                |
|    - *vscr:  virtuelni ekran velicine MAX_CHARS po kom se `crta' |
|                                                                  |
|  Opcije:                                                         |
|    - HEX_REPLACE_SPEC_CHAR:                                      |
|          Karakter kojim se zamenjuju neki specijalni znakovi     |
|          ('\r','\n','\t' i '\0') u preview koloni.  [ch|0|'\0']  |
|                                                                  |
|    - HEX_DELIMITER:                                              |
|          Karakter koji se ispisuje posle 8. kolone srednjeg      |
|          dela ekrana (na slici '-').  [ch|0|'\0']                |
|                                                                  |
`-----------------------------------------------------------------*/

void dump_page(FILE *file, long offset, byte *vscr)
{
/*

 00000000001111111111222222222233333333334444444444555555555566666666667777777777
 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+--------------------------------------------------------------------------------+
|00000000:  61 62 63 64 65 66 67 68-69 6A 6B 6C 6D 6E 6F 70   abcdefghijklmnop   |
+--------------------------------------------------------------------------------+

*/
    byte raw[MAX_HEXCHARS], *buf = raw;
    byte *p, n, ch, i;
    unsigned cnt;

    memset(vscr, ' ', MAX_CHARS);
    if (fseek(file, offset, SEEK_SET))
        error_me("Error seeking in file!");

    cnt = fread(&raw, sizeof(byte), MAX_HEXCHARS, file);
    while (cnt) {
        long pos = offset;
        for (p = vscr + 8, *p-- = ':'; p >= vscr; p--) {
            if ((ch = (pos & 0x0F)) > 9)
                ch += 7;
            *p = ch + '0';
            pos >>= 4;
        }

        vscr += 11;     /* vscr sada pokazuje na pocetak hex dela reda (16x3 bajta) */
        p = vscr + 50;  /* p sada pokazuje na pocetak preview sekcije */

        for (i = 0; cnt && i < 16; i++, cnt--) {
            ch = *buf++;

    #if HEX_REPLACE_SPEC_CHAR
            if (ch == '\r' || ch == '\n' || ch == '\t' || ch == '\0')
                *p++ = HEX_REPLACE_SPEC_CHAR;
            else
    #endif
                *p++ = ch;      /* preview kolona */

            if ((n = (ch >> 4)) > 9)
                n += 7;
            *vscr++ = n + '0';
            if ((n = (ch & 0x0F)) > 9)
                n += 7;
            *vscr = n + '0';
            vscr += 2;          /* preskoci i space */
        }                       /* za svaki red */

    #if HEX_DELIMITER
        if (i >= 9)
            *(vscr - (i - 8) * 3 - 1) = HEX_DELIMITER;
    #endif
        offset += 0x10;  /* imamo 16 hex vrednosti po redu */
        vscr += 21;      /* predji u sledeci red */
    }                    /* end while */
}

/*-----------------------------------------------------------------------.
|  Ispisivanje broja stranica i moda - formatiranje naslovne linije.     |
|                                                                        |
|  Parametri:                                                            |
|    - *buf:     buffer (min.) duzine TITLE_LEN koji se formatira        |
|    - curr:     broj trenutne stranice (min. 1)                         |
|    - total:    ukupan broj stranica (min. 1)                           |
|    - mode:     MODE_TEXT, MODE_BIN ili MODE_HEX                        |
|                                                                        |
|  Napomena:                                                             |
|    Citav naslov (duzine TITLE_LEN bajtova) se kopira samo pri prvom    |
|    pozivu f-je, sto ce reci da se u toku programa ne menjaju prvih     |
|    TITLE_LEN bajtova virtuelnog ekrana. Ne koriste se globalne         |
|    promenljive.                                                        |
`-----------------------------------------------------------------------*/

void write_title(byte *buf, unsigned curr, unsigned total, int mode)
{
/*

 00000000001111111111222222222233333333334444444444555555555566666666667777777777
 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+--------------------------------------------------------------------------------+
|atype v1.17b   Copyright (c) 2001. by -=KKF=-    mode: t b h  page: 00001/00001 |
+--------------------------------------------------------------------------------+

*/
    static byte const *title = logo;  /* logo je vidljiv i izvan f-je zbog asm blokova */
    byte *pcurr, *ptot;

    if (title) {
        memcpy(buf, title, 80); /* nacrtaj logo (ovo se radi samo jednom) */
        title = NULL;           /* resetuj zbog sledeceg poziva f-je */
    } else
        buf[55] = 't', buf[57] = 'b', buf[59] = 'h';

    /* Ovde se koristi cinjenica da MODE_TEXT, MODE_BIN i MODE_HEX
       imaju vrednosti 0,1 i 2 respektivno. */
    buf[55 + mode * 2] -= 'a' - 'A';

    /*  Konvertuj brojeve u njihove string reprezentacije. Ako je broj veci
        od 99999 (akko je int 32-bitni) - nece se lepo prikazati. (imamo
        samo 5 kolona na raspolaganju)  */
    for (pcurr = buf + 72, ptot = buf + 78; pcurr != buf + 67;) {
        *pcurr-- = curr % 10 + '0';
        *ptot-- = total % 10 + '0';
        curr /= 10;
        total /= 10;
    }
}

/*------------------------------------------------------------------------.
|  Kreiranje liste `stranica'.                                            |
|                                                                         |
|  Parametri:                                                             |
|    - *file:    fajl koji se cita                                        |
|    - **first:  adresa promenljive koja ce primiti ptr na prvu stranicu  |
|    - **last:                   -//-                 poslednju stranicu  |
|    - mode:     MODE_TEXT ili MODE_BIN (MODE_HEX nije dozvoljen)         |
|                                                                         |
|  Povratne vrednosti:                                                    |
|    - vraca se ukupan broj stranica (u slucaju praznog fajla: 1!)        |
|    - vracaju se i pointeri na prvu i poslednju stranicu                 |
|                                                                         |
|  Napomena:                                                              |
|    F-ja kreira listu stranica (dvostruko povezana lista) koju kasnije,  |
|    izvan f-je treba i osloboditi (npr. sa `delete_pagelist(first)').    |
|                                                                         |
|    Broj bajtova potrebnih za prikazivanje odredjene stranice se         |
|    izracunava zavisno od izabranog moda (MODE_TEXT ili MODE_BIN):       |
|                                                                         |
|  +===========+                                                          |
|  | MODE_TEXT |                                                          |
|  +===========+                                                          |
|    <CR-XX>    - vraca kursor u prvu kolonu tekuceg reda                 |
|    <CR-LF>    - prelazi se u novi red (prva kolona)                     |
|    \n (0x0A)                                                            |
|      #if LF_IS_NEWLINE                                                  |
|               - prelazi se u novi red (prva kolona)                     |
|      #else                                                              |
|               - pomera kursor red nize (broj kolone se ne menja!)       |
|      #endif                                                             |
|    \t (0x09)  - poravnava tekst na n*TABSIZE granicu                    |
|    \a (0x07)                                                            |
|      #if RING_A_BELL                                                    |
|               - `zvoni', ali ne ispisuje karakter                       |
|      #else                                                              |
|               - prikazuje '\a' kao obican karakter                      |
|      #endif                                                             |
|    \b (0x08)  - pomera kursor jedno mesto ulevo (ako je to izvodljivo)  |
|                                                                         |
|  +===========+                                                          |
|  | MODE_BIN  |                                                          |
|  +===========+                                                          |
|    <CR-XX>    - prikazuje '\r' kao obican karakter                      |
|    <CR-LF>    - prelazi se u novi red (prva kolona)                     |
|    \n (0x0A)                                                            |
|      #if LF_IS_NEWLINE                                                  |
|               - prelazi se u novi red (prva kolona)                     |
|      #else                                                              |
|               - prikazuje '\n' kao obican karakter                      |
|      #endif                                                             |
|    \t (0x09)  - poravnava tekst na n*TABSIZE granicu                    |
|    \a (0x07)  - prikazuje '\a' kao obican karakter                      |
|    \b (0x08)  - prikazuje '\b' kao obican karakter                      |
|                                                                         |
`------------------------------------------------------------------------*/

unsigned create_pagelist(FILE *file, page **first, page **last, int mode)
{
    int ch;          /*  karakter koji se cita                 */
    long foff;       /*  file offset                           */
    page *pg;        /*  stranica koja se dodaje u listu       */
    unsigned nvis;   /*  broj vidljivih karaktera (po strani)  */
    unsigned nread;  /*  broj procitanih bajtova  (po strani)  */
    unsigned numba;  /*  broj procitanih stranica              */

    nvis = nread = foff = 0;
    pg = NULL;  /* lista se kreira pri prvom pozivu `add_page()' f-je */

    if (fseek(file, 0L, SEEK_SET))
        error_me("Error seeking in file!");

    while ((ch = fgetc(file)) != EOF) {
        switch (ch) {
          case '\r':
              if ((ch = fgetc(file)) == '\n') {
                  if (nvis >= SCR_COLS * (SCR_ROWS - 1)) { /* da li smo u poslednjem redu ? */
                      pg = add_page(pg, foff, nread);   /* nread jos uvek nije inkrementiran! */
                      foff += nread + 2;  /* procitani CR-LF se ne pridruzuju nijednoj stranici */
                      nvis = nread = 0; continue;
                  } else {
                      nvis += SCR_COLS - nvis % SCR_COLS;  /* predji u sledeci red */
                      ++nread;  /* procitali smo i <LF> */
                  }
              } else {  /* nije <CR-LF> kombinacija */
                  if (mode == MODE_BIN)
                      ++nvis;   /* '\r' se prikazuje kao obican karakter */
                  else
                      nvis -= nvis % SCR_COLS;  /* vrati kursor na pocetak linije */
                  ungetc(ch, file);  /* vrati poslednji procitani karakter u buffer */
              }
              break;

          case '\n':
              if (nvis >= SCR_COLS * (SCR_ROWS - 1)) {  /* ako smo u poslednjem redu */
                  pg = add_page(pg, foff, nread);  /* nread jos uvek nije inkrementiran! */
                  foff += nread + 1;  /* procitani LF se ne pridruzuje nijednoj stranici */
                  nvis = nread = 0; continue;
              } else {  /* ne, jos uvek nismo u poslednjem (vidljivom) redu */

    #if LF_IS_NEWLINE
                  nvis += SCR_COLS - nvis % SCR_COLS;   /* predji u sledeci red */
    #else
                  if (mode == MODE_BIN)
                      ++nvis;   /* '\n' se prikazuje kao obican karakter */
                  else
                      nvis += SCR_COLS; /* predji u sledeci red (kolona se ne menja) */
    #endif
              }
              break;

          case '\t':
          {
              /* U slucaju prelamanja u sledeci red (akko TABSIZE != 8), f-ja koja
                 `crta' ekran popunjava belinama ostatak tekuceg reda. Promenljiva
                 `nvis' se testira ispod ovog switch-a. */
              int indent = TABSIZE - nvis % TABSIZE;

    #if TABSIZE != 8
              if ((nvis % SCR_COLS + indent) > SCR_COLS)
                  nvis += SCR_COLS - nvis % SCR_COLS;   /* predji u sledeci red */

              else
    #endif
                  nvis += indent; /* nema prelamanja */
          }
              break;


    #if RING_A_BELL
          case '\a':
              if (mode == MODE_BIN)
                  ++nvis;  /*  dodaj u listu vidljivih karaktera */
              break;       /*  preskoci ++nvis (default iskaz) u TEXT modu  */
    #endif

          case '\b':
              if (mode == MODE_TEXT) {
                  if (nvis % SCR_COLS)  /* ako nismo na pocetku reda */
                      --nvis;   /* vrati kursor za jedno mesto ulevo */
                  break;        /* i preskoci ++nvis */
              }

              /* u bin. modu samo ispisi karakter (ovde nema break-a) */

          default:
              ++nvis;  /* povecaj broj vidljivih karaktera */
              break;   /* (procitan je `obican' karakter) */

        }  /* end switch */

        ++nread;  /* povecaj broj procitanih (raw) bajtova */
        if (nvis == MAX_CHARS) {
            pg = add_page(pg, foff, nread);
            foff += nread; nvis = nread = 0;
        }
    }     /* end while */


    /* U slucaju da ekran nije maksimalno popunjen (nread!=0) ili ako je lista
       prazna (pg==NULL) - dodaj stranicu. Na ovaj nacin kontrolisemo izuzetke:
       maksimalno popunjenu stranicu i prazan fajl. */

    if (nread != 0 || pg == NULL)
        pg = add_page(pg, foff, nread);

    *last = pg;
    for (numba = 1; pg->prev; pg = pg->prev)
        ++numba;

    *first = pg;
    return numba;
}

/*-------------------------------------------------------------------------.
|                                                   __   __   ____         |
|                                                  / /__/ /__/ __/         |
|  Advanced Type  v1.17b                          /  '_/  '_/ _/           |
|  Copyright (c) 2001. by -=KKF=-                /_/\_Y_/\_Y_/  -=KKF=-    |
|                                                                          |
|                                                                          |
|  ATYPE je zamisljen kao `naprednija' verzija DOS-ove `type' komande.     |
|  Program bi trebalo da podrzava skrolovanje (sadrzaja) ekrana pomocu     |
|  PgUp, PgDn, Home i End tastera. Jos jedno `type' ogranicenje je 0x1A    |
|  (ASCII26) tj. DOS_EOF karakter koji je pod DOS-om oznacavao kraj fajla. |
|  Kada type naleti na '\x1A' karakter, on jednostavno - stane. Posto ovaj |
|  program treba da podrzava `skakutanje' po fajlu, resenje je, naravno,   |
|   _uvek_ otvoriti ulazni fajl u binarnom rezimu (_O_BINARY) i `rucno'    |
|   citati bajt-po-bajt.                                                   |
|                                                                          |
|  Moglo bi se reci da je TEXT mod (_O_TEXT) veoma nesrecno resen u C-u:   |
|   - pod DOS-om, 0x1A predstavlja kraj fajla!                             |
|   - CR-LF (tj. `\r\n') kombinacije se na ulazu pretvaraju u LF ('\n'),   |
|     a na izlazu se svaki LF `prosiruje' u CR-LF. Dok to s jedne strane   |
|     olaksava detektovanje oznake za novu liniju ('\n'), sa druge strane, |
|     postaje nemoguce `uloviti' samostalan LF ('\n') karakter na ulazu!   |
|   - po ANSI standardu, u TEXT modu nije precizno definisano ponasanje    |
|     `fseek()' funkcije, upravo zbog <CR-LF> translacije, sto samo        |
|     dodatno otezava stvari!                                              |
|                                                                          |
|  Evo nekih nedostataka standardne ANSI C biblioteke:                     |
|   - ne postoji clearscreen() f-ja                                        |
|   - ne postoje f-je za rad sa konzolom (putch()/cprintf() nisu ANSI)     |
|   - ne postoje f-je za slobodno `crtanje' po tekstualnom ekranu          |
|                                                                          |
|  Iz tog razloga su sledece f-je implementirane ili kao makroi (koji      |
|  pozivaju odgovarajuce bibliotecke f-je odredjenog kompajlera) ili kao   |
|  posebne, system-specific funkcije (npr. za Win32):                      |
|                                                                          |
|     atype_initapp()       -  poziva se na pocetku programa               |
|     atype_clearscreen()   -  brise ekran                                 |
|     atype_settextpos()    -  menja poziciju ispisa teksta (kursora)      |
|     atype_showcursor()    -  prikazuje/sakriva kursor                    |
|     atype_waitvretrace()  -  ceka vertikalni retrace (ne i pod Win32)    |
|     atype_flipscreen()    -  trenutno kopira virtuelni na vidljivi ekran |
|                                                                          |
|  Program je testiran (i kompajlira se bez problema) sa sledecim          |
|  MSDOS/Win32 C kompajlerima:                                             |
|                                                                          |
|   - Turbo C (ovo je bio uslov u zadatku)                                 |
|   - Watcom 16-bit C                                                      |
|   - M$VC++ v6.0                                                          |
|                                                                          |
|  Mogu se koristiti i drugi Win32 kompajleri, samo je potrebno definisati |
|  `__WIN32__' makro.                                                      |
|                                                                          |
|  Napomena:                                                               |
|    Za razliku od vecine zadataka (a suprotno `Uputstvu', prof. D.V),     |
|    ATYPE ne posmatra STDIN kao ulazni fajl, vec mora biti pozvan iz      |
|    komandne linije na sledeci nacin:                                     |
|                                                                          |
|          atype filename       (a _ne_ sa `atype <filename'!)             |
|                                                                          |
`-------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
    char *fname;                /*   ime datoteke                            */
    FILE *file;                 /*   fajl koji prikazujemo                   */
    page *first_page;           /*   pointer na prvu stranicu u listi        */
    page *last_page;            /*   pointer na poslednju stranicu u listi   */
    page *pg;                   /*   pointer na trenutnu stranicu            */
    byte *vscreen;              /*   virtuelni ekran velicine SCREEN_SIZE    */
    unsigned curr_page;         /*   broj trenutne stranice (min. 1)         */
    unsigned npages;            /*   ukupan broj stranica u TEXT/BIN modu    */
    unsigned nhex_pages;        /*   ukupan broj stranica u HEX modu (const) */
    LIST_TYPE got_list;         /*   tip liste koju vec imamo kreiranu       */
    VIEW_MODE mode = MODE_TEXT; /*   trenutni mod (default: MODE_TEXT)       */
    long fsize;                 /*   duzina fajla                            */
    long hexpos;                /*   trenutni file offset u HEX modu         */
    int key;                    /*   scan code pritisnutog tastera           */
    const char usage[] = "Usage: atype [-t|-b|-h] filename\n";

    atype_initapp();

    if (argc == 1 || argc > 3) {
        fprintf(stderr, usage);
        return EXIT_FAILURE;
    }

    fname = argv[argc - 1];  /* ime fajla je uvek poslednji argument */

    /* tako je ovde, npr. za MODE_HEX dozvoljen svaki od sledecih parametara:
       `-h', `-heX', `/HexMode', `/H', `-how-lame' itd. */

    if (argc == 3) {
        char ch = **++argv;
        if (ch == '-' || ch == '/')
            switch (tolower(*++*argv)) {
              case 't': mode = MODE_TEXT; break;
              case 'b': mode = MODE_BIN;  break;
              case 'h': mode = MODE_HEX;  break;
              default:
                  fprintf(stderr, "atype: Invalid option!\n%s", usage);
                  return EXIT_FAILURE;
            }
    }

    /* kursor se, nazalost, uvek vidi u DOS _prozoru_ pod Win32!
       (ali ne i u fullscreen modu) */
    atype_settextpos(SCREEN_COLS - 1, SCREEN_ROWS - 1);
    atype_showcursor(CUR_OFF);

    /* fajl uvek otvaramo u binarnom rezimu - vise
       nema DOS_EOF (0x1A) ogranicenja! */
    if ((file = fopen(fname, "rb")) == NULL)
        error_me("Error opening file!");

    fseek(file, 0L, SEEK_END);
    fsize = ftell(file);
    nhex_pages = fsize / MAX_HEXCHARS + (fsize == 0 || fsize % MAX_HEXCHARS != 0);

    /* alociraj dovoljno memorije za virtuelni ekran */
    vscreen = (byte *) xmalloc(SCREEN_SIZE * sizeof(byte));

    if (mode != MODE_HEX) {
        npages = create_pagelist(file, &first_page, &last_page, mode);
        load_page(file, pg = first_page, vscreen + TITLE_LEN, mode);
        got_list = mode;
        atype_flipscreen(vscreen, curr_page = 1, npages, mode);
    } else {
        dump_page(file, hexpos = 0L, vscreen + TITLE_LEN);
        got_list = LIST_NONE;   /* na pocetku nemamo nijednu listu */
        atype_flipscreen(vscreen, curr_page = 1, nhex_pages, mode);
    }

    while ((key = getch()) != ESC_KEY) {
        switch (key = toupper(key)) {

          case 0x00:
          case 0xE0:
              switch (getch()) {

                case PGUP_KEY:
                    if (mode == MODE_HEX) {
                        if (hexpos != 0L) {
                            dump_page(file, hexpos -= MAX_HEXCHARS, vscreen + TITLE_LEN);
                            atype_flipscreen(vscreen, --curr_page, nhex_pages, mode);
                        }
                    } else if (pg != first_page) {
                        load_page(file, pg = pg->prev, vscreen + TITLE_LEN, mode);
                        atype_flipscreen(vscreen, --curr_page, npages, mode);
                    }
                    break;

                case PGDN_KEY:
                    if (mode == MODE_HEX) {
                        if (hexpos + MAX_HEXCHARS < fsize) {
                            dump_page(file, hexpos += MAX_HEXCHARS, vscreen + TITLE_LEN);
                            atype_flipscreen(vscreen, ++curr_page, nhex_pages, mode);
                        }
                    } else if (pg != last_page) {
                        load_page(file, pg = pg->next, vscreen + TITLE_LEN, mode);
                        atype_flipscreen(vscreen, ++curr_page, npages, mode);
                    }
                    break;

                case HOME_KEY:
                    if (mode == MODE_HEX) {
                        if (hexpos != 0L) {
                            dump_page(file, hexpos = 0L, vscreen + TITLE_LEN);
                            atype_flipscreen(vscreen, curr_page = 1, nhex_pages, mode);
                        }
                    } else if (pg != first_page) {
                        load_page(file, pg = first_page, vscreen + TITLE_LEN, mode);
                        atype_flipscreen(vscreen, curr_page = 1, npages, mode);
                    }
                    break;

                case END_KEY:
                    if (mode == MODE_HEX) {
                        if (hexpos + MAX_HEXCHARS < fsize) {
                            dump_page(file, hexpos = (fsize % MAX_HEXCHARS) ? fsize -
                                fsize % MAX_HEXCHARS : fsize - MAX_HEXCHARS, vscreen + TITLE_LEN);
                            atype_flipscreen(vscreen, curr_page = nhex_pages, nhex_pages, mode);
                        }
                    } else if (pg != last_page) {
                        load_page(file, pg = last_page, vscreen + TITLE_LEN, mode);
                        atype_flipscreen(vscreen, curr_page = npages, npages, mode);
                    }
                    break;
              }             /* end switch spec. key */
              break;

          case 'T':
          case 'B':
          {
              VIEW_MODE old_mode = mode;
              mode = MODE_TEXT + (key == 'B');
              if (old_mode != mode) {
                  if (got_list != mode) {
                      if (got_list != LIST_NONE)
                          delete_pagelist(first_page);
                      npages = create_pagelist(file, &first_page, &last_page, mode);
                      got_list = mode;
                  }
                  load_page(file, pg = first_page, vscreen + TITLE_LEN, mode);
                  atype_flipscreen(vscreen, curr_page = 1, npages, mode);
              }
          }
          break;

          case 'H':
              if (mode != MODE_HEX) {
                  dump_page(file, hexpos = 0L, vscreen + TITLE_LEN);
                  atype_flipscreen(vscreen, curr_page = 1, nhex_pages, mode = MODE_HEX);
              }
              break;
        }             /* end switch #1 */
    }                 /* end while */

    if (got_list != LIST_NONE)
        delete_pagelist(first_page);
    free(vscreen);
    fclose(file);

    atype_settextpos(0, 0);
    atype_showcursor(CUR_ON);
    atype_clearscreen();
    cprintf("Thank you for using atype v1.17b.\r\nHave a nice day! :-)\r\n");

    return EXIT_SUCCESS;
}
