
#ifndef _ATYPE_H
#define _ATYPE_H

#include <stdio.h>
#include <stdarg.h>

#ifdef __TURBOC__
#  define __TURBOFOLK__
#elif defined (_MSC_VER) || defined (_WIN32)
#  define __WIN32__
#endif

#define BACK_BLACK           0x00
#define BACK_BLUE            0x10
#define BACK_GREEN           0x20
#define BACK_CYAN            0x30
#define BACK_RED             0x40
#define BACK_MAGENTA         0x50
#define BACK_BROWN           0x60
#define BACK_LIGHTGREY       0x70

#define FORE_BLACK           0x00
#define FORE_BLUE            0x01
#define FORE_GREEN           0x02
#define FORE_CYAN            0x03
#define FORE_RED             0x04
#define FORE_MAGENTA         0x05
#define FORE_BROWN           0x06
#define FORE_LIGHTGREY       0x07
#define FORE_DARKGREY        0x08
#define FORE_LIGHTBLUE       0x09
#define FORE_LIGHTGREEN      0x0A
#define FORE_LIGHTCYAN       0x0B
#define FORE_LIGHTRED        0x0C
#define FORE_LIGHTMAGENTA    0x0D
#define FORE_YELLOW          0x0E
#define FORE_WHITE           0x0F

#define TEXT_COLOR (BACK_BLACK | FORE_LIGHTGREY)

typedef enum { MODE_TEXT, MODE_BIN, MODE_HEX } VIEW_MODE;
typedef enum { LIST_TEXT, LIST_BIN, LIST_NONE } LIST_TYPE;
typedef unsigned char byte;

typedef struct tagpage {
    long offset;
    unsigned cnt;
    struct tagpage *prev;
    struct tagpage *next;
} page;

/* broj redova i kolona potrebnih za ispis strane */
#define SCR_ROWS           24
#define SCR_COLS           80

/* maksimalni broj karaktera koji staju na jednu
   stranu (naslovna linija se ne racuna) */
#define MAX_CHARS          (SCR_ROWS*SCR_COLS)

/* koliko je raw bajtova moguce prikazati na HEX ekranu */
#define MAX_HEXCHARS       (SCR_ROWS*16)

/* dimenzije ekrana (one su tu samo zbog (ne)preglednosti
   jer je `atype' ionako pisan za `fiksni' ekran: 80x25) */
#define SCREEN_ROWS        25
#define SCREEN_COLS        80
#define SCREEN_SIZE        (80*25)

/* scan kodovi (nama) bitnih tastera */
#define ESC_KEY            0x1B
#define PGUP_KEY           0x49
#define PGDN_KEY           0x51
#define HOME_KEY           0x47
#define END_KEY            0x4F

/* prototipovi f-ja */
void     *xmalloc(size_t);
void      error_me(const char *, ...);
page     *add_page(page *, long, unsigned);
unsigned  create_pagelist(FILE *, page **, page **, int);
void      delete_pagelist(page *);
void      load_page(FILE *, const page *, byte *, int);
void      dump_page(FILE *, long, byte *);
void      write_title(byte *, unsigned, unsigned, int);

extern const byte logo[80];
extern const byte TITLE_CLR;
extern const byte TITLE_LEN;

#endif
