
#include "atype.h"
#include "misc.h"
#include "options.h"

#ifdef __WATCOMC__

void atype_flipscreen(byte *buf, unsigned curr, unsigned total, int mode)
{
    write_title(buf, curr, total, mode);
    atype_waitvretrace();
    _atype_flipscreen(buf);
}

#elif defined __TURBOFOLK__

#include <dos.h>

void atype_showcursor(unsigned mode)
{
    union REGS reg;
    reg.h.ah = 1;
    reg.x.cx = mode;
    int86(0X10, &reg, &reg);  /* tc ne podrzava _asm blokove */
}

void atype_waitvretrace()
{
    while (inp(0x3DA) & 0x08);
    while (!(inp(0x3DA) & 0x08));
}

void atype_flipscreen(byte *buf, unsigned curr, unsigned total, int mode)
{
    char far *scr = (char far *) 0xB8000000;
    int i;

    write_title(buf, curr, total, mode);
    atype_waitvretrace();

    for (i = 0; i < TITLE_LEN; i++) {
        *scr++ = *buf++;
        *scr++ = TITLE_CLR;
    }

    for (; i < SCREEN_SIZE; i++) {
        *scr++ = *buf++;
        *scr++ = TEXT_COLOR;
    }
}

#elif defined __WIN32__

#include <windows.h>
#include <stdlib.h>

static HANDLE hStdOut;

void atype_clearscreen()
{
    DWORD num;
    COORD coord = { 0, 0 };
    FillConsoleOutputAttribute(hStdOut, FOREGROUND_RED |
        FOREGROUND_GREEN | FOREGROUND_BLUE, SCREEN_SIZE, coord, &num);
    FillConsoleOutputCharacter(hStdOut, ' ', SCREEN_SIZE, coord, &num);
}

void atype_showcursor(int mode)
{
    CONSOLE_CURSOR_INFO cinfo;
    static CONSOLE_CURSOR_INFO old = { -1, -1 };

    if (old.bVisible == -1 || old.dwSize == -1)
        GetConsoleCursorInfo(hStdOut, &old);

    cinfo.bVisible = mode;
    cinfo.dwSize = old.dwSize;  /* procenat popunjenosti celije */
    SetConsoleCursorInfo(hStdOut, &cinfo);
}

/* koordinate pocinju od (0,0) */
void atype_settextpos(int x, int y)
{
    COORD pos = { x, y };
    SetConsoleCursorPosition(hStdOut, pos);
}

void atype_flipscreen(byte *buf, unsigned curr, unsigned total, int mode)
{
    CHAR_INFO *p, scr[SCREEN_SIZE];
    COORD size = { SCREEN_COLS, SCREEN_ROWS }, orig = { 0, 0 };
    SMALL_RECT rc = { 0, 0, SCREEN_COLS - 1, SCREEN_ROWS - 1 };
    byte *last = &buf[TITLE_LEN - 1];

    write_title(buf, curr, total, mode);

    for (p = scr; buf <= last; buf++, p++) {
        p->Char.AsciiChar = (char) *buf;
        p->Attributes = TITLE_CLR;
    }

    for (last = &buf[MAX_CHARS - 1]; buf <= last; buf++, p++) {
        p->Char.AsciiChar = (char) *buf;
        p->Attributes = FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
    }

    WriteConsoleOutput(hStdOut, scr, size, orig, &rc);
}

void atype_initapp()
{
    if ((hStdOut = GetStdHandle(STD_OUTPUT_HANDLE)) == INVALID_HANDLE_VALUE)
        exit(EXIT_FAILURE);
}

#endif
