
#ifndef _MISC_H
#define _MISC_H

#include "atype.h"

#ifdef __TURBOFOLK__

  #include <fcntl.h>
  #define CUR_ON  0x0607
  #define CUR_OFF 0x2000
  #define atype_clearscreen() do { atype_waitvretrace(); clrscr(); } while (0)
  #define atype_initapp() ((void)0)
  #define atype_settextpos(x,y)  gotoxy((x)+1,(y)+1)

  void atype_waitvretrace();
  void atype_showcursor(unsigned);
  void atype_flipscreen(byte *, unsigned, unsigned, int);

#elif defined __WATCOMC__

  #define CUR_ON  0x0607
  #define CUR_OFF 0x2000

  #if USE_WATCOMS_LIB
    #include <graph.h>
    #define atype_showcursor(mode) _settextcursor(mode)
    #define atype_settextpos(x,y)  _settextposition((y)+1,(x)+1)
    #define atype_clearscreen() do { atype_waitvretrace();        \
                                     _clearscreen(_GCLEARSCREEN); \
                                   } while (0)
  #else
    #define atype_clearscreen() do { atype_waitvretrace(); \
                                     _atype_clearscreen(); \
                                   } while(0)
    #define atype_showcursor(mode) _atype_showcursor(mode)
    /* ako parametri nisu konstantni - koristiti dl i dh umesto dx */
    #define atype_settextpos(x,y) _atype_settextpos((((byte)(y))<<8)|((byte)(x)))

    /*  Watcom && `#pragma aux' blokovi
     *
     *  Ovo je najbolji nacin za definisanje asm blokova za Watcom C/C++
     *  kompajler. Bolji je od `obicnih', _asm {...} blokova jer
     *  se, zavisno od duzine, rutine inline-uju u kod, a potrebno
     *  je i eksplicitno navesti registre koje rutina menja, sto samo
     *  pomaze Watcom-u da generise bolji kod. Kod _asm {...} blokova to
     *  nije slucaj - cesto se citava grupa registara cuva na steku i
     *  restaurira posle asm bloka! Ovde je moguce zadati i parametre u
     *  zeljenim registrima, pa ce Watcom tako konstruisati pozive (bolje
     *  je cak i od __fastcall calling konvencije). Povratna vrednost se
     *  ne mora vracati u (E)AX registru.
     */
    extern void _atype_clearscreen();
    #pragma aux _atype_clearscreen=       \
              "mov ax, 0b800h"            \
              "mov es, ax"                \
              "xor di, di"                \
              "mov cx, 07d0h"             \
              "mov ax, 0720h"             \
              "cld"                       \
              "rep stosw"                 \
              modify [es ax cx di];

    extern void _atype_showcursor(unsigned mode);
    #pragma aux _atype_showcursor=        \
              "mov ah, 01h"               \
              "int 10h"                   \
              parm [cx]                   \
              modify [ah cx];

    extern void _atype_settextpos(unsigned rowcol);
    #pragma aux _atype_settextpos=        \
              "mov ah, 02h"               \
              "mov bh, 00h"               \
              "int 10h"                   \
              parm [dx]                   \
              modify [ah bh dx];
  #endif

  #define atype_initapp() ((void)0)
  void atype_flipscreen(byte *, unsigned, unsigned, int);

  extern void atype_waitvretrace();
  #pragma aux atype_waitvretrace=       \
            "mov dx,3DAh"               \
     "@1:    in al,dx"                  \
            "and al,08h"                \
            "jnz @1"                    \
     "@2:    in al,dx"                  \
            "and al,08h"                \
            "jz  @2"                    \
            modify [ax dx];

  extern void _atype_flipscreen(const byte *buf);
  #pragma aux _atype_flipscreen=        \
            "mov ax, 0b800h"            \
            "mov es, ax"                \
            "xor di, di"                \
            "mov ah, TITLE_CLR"         \
            "mov cl, TITLE_LEN"         \
   "@title:  mov al, byte ptr [si]"     \
            "mov word ptr es:[di], ax"  \
            "inc si"                    \
            "add di,2"                  \
            "dec cl"                    \
            "jnz @title"                \
            "mov ah, 07h"               \
            "mov cx, 0780h"             \
    "@text:  mov al, byte ptr [si]"     \
            "mov word ptr es:[di], ax"  \
            "inc si"                    \
            "add di,2"                  \
            "dec cx"                    \
            "jnz @text"                 \
            parm [si]                   \
            modify [es ax cx si di];

#elif defined __WIN32__

  #include <windows.h>
  #define CUR_ON   1
  #define CUR_OFF  0
  void atype_initapp();
  void atype_clearscreen();
  void atype_showcursor(int);
  void atype_settextpos(int, int);
  void atype_flipscreen(byte *, unsigned, unsigned, int);

#endif

#endif
